@@ -1,9 +1,11 @@
+mod build_info_abi;
 mod contract_abi;
 mod endpoint_abi;
 mod type_abi;
 mod type_description;
 mod type_description_container;
 
+pub use build_info_abi::*;
 pub use contract_abi::*;
 pub use endpoint_abi::*;
 pub use type_abi::*;
