@@ -1,38 +1,37 @@
 use super::{set_mapper, SetMapper, StorageClearable, StorageMapper};
-use crate::api::{ErrorApi, StorageReadApi, StorageWriteApi};
-use crate::storage;
-use crate::types::BoxedBytes;
-use alloc::vec::Vec;
+use crate::{
+    api::{ErrorApi, ManagedTypeApi, StorageReadApi, StorageWriteApi},
+    storage::{self, StorageKey},
+};
 use core::marker::PhantomData;
-use elrond_codec::{TopDecode, TopEncode};
+use elrond_codec::{NestedDecode, NestedEncode, TopDecode, TopEncode};
 
 const MAPPED_STORAGE_VALUE_IDENTIFIER: &[u8] = b".storage";
 type Keys<'a, SA, T> = set_mapper::Iter<'a, SA, T>;
 
-#[deprecated]
 pub struct MapStorageMapper<SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     api: SA,
-    main_key: BoxedBytes,
+    base_key: StorageKey<SA>,
     keys_set: SetMapper<SA, K>,
     _phantom: core::marker::PhantomData<V>,
 }
 
 impl<SA, K, V> StorageMapper<SA> for MapStorageMapper<SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode,
     V: StorageMapper<SA> + StorageClearable,
 {
-    fn new(api: SA, main_key: BoxedBytes) -> Self {
+    fn new(api: SA, base_key: StorageKey<SA>) -> Self {
         Self {
             api: api.clone(),
-            main_key: main_key.clone(),
-            keys_set: SetMapper::<SA, K>::new(api, main_key),
+            base_key: base_key.clone(),
+            keys_set: SetMapper::<SA, K>::new(api, base_key),
             _phantom: PhantomData,
         }
     }
@@ -40,8 +39,8 @@ where
 
 impl<SA, K, V> StorageClearable for MapStorageMapper<SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode,
     V: StorageMapper<SA> + StorageClearable,
 {
     fn clear(&mut self) {
@@ -52,28 +51,22 @@ where
     }
 }
 
-pub fn top_encode_to_vec<T: TopEncode>(obj: &T) -> Vec<u8> {
-    let mut bytes = Vec::<u8>::new();
-    obj.top_encode(&mut bytes).unwrap();
-    bytes
-}
-
 impl<SA, K, V> MapStorageMapper<SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode,
     V: StorageMapper<SA> + StorageClearable,
 {
-    fn build_named_key(&self, name: &[u8], key: &K) -> BoxedBytes {
-        let bytes = top_encode_to_vec(&key);
-        BoxedBytes::from_concat(&[self.main_key.as_slice(), name, &bytes])
+    fn build_named_key(&self, name: &[u8], key: &K) -> StorageKey<SA> {
+        let mut named_key = self.base_key.clone();
+        named_key.append_bytes(name);
+        named_key.append_item(key);
+        named_key
     }
 
     fn get_mapped_storage_value(&self, key: &K) -> V {
-        <V as storage::mappers::StorageMapper<SA>>::new(
-            self.api.clone(),
-            self.build_named_key(MAPPED_STORAGE_VALUE_IDENTIFIER, key),
-        )
+        let key = self.build_named_key(MAPPED_STORAGE_VALUE_IDENTIFIER, key);
+        <V as storage::mappers::StorageMapper<SA>>::new(self.api.clone(), key)
     }
 
     /// Returns `true` if the map contains no elements.
@@ -159,8 +152,8 @@ where
 
 pub struct Iter<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     key_iter: Keys<'a, SA, K>,
@@ -169,8 +162,8 @@ where
 
 impl<'a, SA, K, V> Iter<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     fn new(hash_map: &'a MapStorageMapper<SA, K, V>) -> Iter<'a, SA, K, V> {
@@ -183,8 +176,8 @@ where
 
 impl<'a, SA, K, V> Iterator for Iter<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     type Item = (K, V);
@@ -201,8 +194,8 @@ where
 
 pub struct Values<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     key_iter: Keys<'a, SA, K>,
@@ -211,8 +204,8 @@ where
 
 impl<'a, SA, K, V> Values<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     fn new(hash_map: &'a MapStorageMapper<SA, K, V>) -> Values<'a, SA, K, V> {
@@ -225,8 +218,8 @@ where
 
 impl<'a, SA, K, V> Iterator for Values<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     type Item = V;
@@ -243,8 +236,8 @@ where
 
 pub enum Entry<'a, SA, K: 'a, V: 'a>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     /// A vacant entry.
@@ -258,8 +251,8 @@ where
 /// It is part of the [`Entry`] enum.
 pub struct VacantEntry<'a, SA, K: 'a, V: 'a>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     pub(super) key: K,
@@ -273,8 +266,8 @@ where
 /// It is part of the [`Entry`] enum.
 pub struct OccupiedEntry<'a, SA, K: 'a, V: 'a>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     pub(super) key: K,
@@ -286,8 +279,8 @@ where
 
 impl<'a, SA, K, V> Entry<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + Clone + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     /// Ensures a value is in the entry by inserting the default if empty, and returns
@@ -325,8 +318,8 @@ where
 
 impl<'a, SA, K, V> Entry<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + Clone + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     /// Ensures a value is in the entry by inserting the default value if empty,
@@ -341,8 +334,8 @@ where
 
 impl<'a, SA, K, V> VacantEntry<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + Clone + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     /// Gets a reference to the key that would be used when inserting a value
@@ -365,8 +358,8 @@ where
 
 impl<'a, SA, K, V> OccupiedEntry<'a, SA, K, V>
 where
-    SA: StorageReadApi + StorageWriteApi + ErrorApi + Clone + 'static,
-    K: TopEncode + TopDecode + Clone + 'static,
+    SA: StorageReadApi + StorageWriteApi + ManagedTypeApi + ErrorApi + Clone + 'static,
+    K: TopEncode + TopDecode + NestedEncode + NestedDecode + Clone + 'static,
     V: StorageMapper<SA> + StorageClearable,
 {
     /// Gets a reference to the key in the entry.
