@@ -1,7 +1,9 @@
 pub mod mappers;
 pub mod protected_keys;
-pub mod storage_get;
-pub mod storage_set;
+mod storage_get;
+mod storage_key;
+mod storage_set;
 
 pub use storage_get::*;
+pub use storage_key::*;
 pub use storage_set::*;
