@@ -1,15 +1,16 @@
 use crate::{
-    abi::TypeAbi,
     api::{Handle, ManagedTypeApi},
     types::{BigUint, ManagedBuffer, ManagedType, ManagedVecItem},
 };
-use alloc::string::String;
 
 use super::{EsdtTokenType, TokenIdentifier};
 
 use elrond_codec::elrond_codec_derive::{NestedDecode, NestedEncode, TopDecode, TopEncode};
 
-#[derive(TopDecode, TopEncode, NestedDecode, NestedEncode, Clone, PartialEq, Debug)]
+use crate as elrond_wasm; // needed by the TypeAbi generated code
+use crate::derive::TypeAbi;
+
+#[derive(TopDecode, TopEncode, NestedDecode, NestedEncode, TypeAbi, Clone, PartialEq, Debug)]
 pub struct EsdtTokenPayment<M: ManagedTypeApi> {
     pub token_type: EsdtTokenType,
     pub token_identifier: TokenIdentifier<M>,
@@ -17,12 +18,6 @@ pub struct EsdtTokenPayment<M: ManagedTypeApi> {
     pub amount: BigUint<M>,
 }
 
-impl<M: ManagedTypeApi> TypeAbi for EsdtTokenPayment<M> {
-    fn type_name() -> String {
-        "EsdtTokenPayment".into()
-    }
-}
-
 impl<M: ManagedTypeApi> EsdtTokenPayment<M> {
     pub fn no_payment(api: M) -> Self {
         EsdtTokenPayment {
