@@ -13,6 +13,7 @@ mod managed_conversion;
 mod managed_default;
 mod managed_multi_result_vec;
 mod managed_multi_result_vec_iter;
+mod managed_ref;
 mod managed_type_trait;
 mod managed_vec;
 mod managed_vec_item;
@@ -30,6 +31,7 @@ pub use managed_conversion::{ManagedFrom, ManagedInto};
 pub use managed_default::ManagedDefault;
 pub use managed_multi_result_vec::ManagedMultiResultVec;
 pub use managed_multi_result_vec_iter::ManagedMultiResultVecIterator;
+pub use managed_ref::{AsManagedRef, ManagedRef};
 pub use managed_type_trait::ManagedType;
 pub use managed_vec::{
     managed_vec_from_slice_of_boxed_bytes, managed_vec_of_buffers_to_arg_buffer, ManagedVec,
